@@ -49,9 +49,6 @@ struct _GoaIdentityServicePrivate
 
   GoaIdentityManager       *identity_manager;
 
-  guint                     realmd_watch;
-  GCancellable             *cancellable;
-
   GHashTable               *watched_client_connections;
   GHashTable               *key_holders;
   GHashTable               *pending_temporary_account_results;
@@ -1602,7 +1599,7 @@ on_identities_listed (GoaIdentityManager *manager,
           goa_warning ("Could not list identities: %s", error->message);
           g_error_free (error);
         }
-      return;
+      goto out;
     }
 
   for (node = identities; node != NULL; node = node->next)
@@ -1610,8 +1607,9 @@ on_identities_listed (GoaIdentityManager *manager,
       GoaIdentity *identity = node->data;
       const char  *principal;
       GoaObject   *object;
+      char        *object_path;
 
-      export_identity (self, identity);
+      object_path = export_identity (self, identity);
 
       principal = goa_identity_get_identifier (identity);
 
@@ -1621,7 +1619,12 @@ on_identities_listed (GoaIdentityManager *manager,
         add_temporary_account (self, identity);
       else
         g_object_unref (object);
+
+      g_free (object_path);
     }
+
+ out:
+  g_object_unref (self);
 }
 
 static void
@@ -1684,7 +1687,7 @@ on_got_client (GoaClient          *client,
   if (self->priv->client == NULL)
     {
       goa_warning ("Could not create client: %s", error->message);
-      return;
+      goto out;
     }
 
   self->priv->accounts_manager = goa_client_get_manager (client);
@@ -1694,16 +1697,19 @@ on_got_client (GoaClient          *client,
   if (self->priv->identity_manager == NULL)
     {
       goa_warning ("Could not create identity manager: %s", error->message);
-      return;
+      goto out;
     }
 
   goa_identity_manager_list_identities (self->priv->identity_manager,
                                         NULL,
                                         (GAsyncReadyCallback)
                                         on_identities_listed,
-                                        self);
+                                        g_object_ref (self));
 
   ensure_credentials_for_accounts (self);
+
+ out:
+  g_object_unref (self);
 }
 
 static void
@@ -1723,7 +1729,7 @@ on_session_bus_acquired (GDBusConnection    *connection,
     goa_client_new (NULL,
                     (GAsyncReadyCallback)
                     on_got_client,
-                    self);
+                    g_object_ref (self));
   }
 }
 
@@ -1755,8 +1761,6 @@ goa_identity_service_activate (GoaIdentityService   *self,
 
   goa_debug ("GoaIdentityService: Activating identity service");
 
-  self->priv->cancellable = g_cancellable_new ();
-
   self->priv->object_manager_server =
     g_dbus_object_manager_server_new ("/org/gnome/Identity");
 
@@ -1786,9 +1790,6 @@ goa_identity_service_deactivate (GoaIdentityService *self)
 {
   goa_debug ("GoaIdentityService: Deactivating identity service");
 
-  if (self->priv->realmd_watch == 0)
-    g_bus_unwatch_name (self->priv->realmd_watch);
-
   if (self->priv->identity_manager != NULL)
     {
       g_signal_handlers_disconnect_by_func (self, on_identity_needs_renewal, self);
@@ -1800,7 +1801,6 @@ goa_identity_service_deactivate (GoaIdentityService *self)
   g_clear_object (&self->priv->object_manager_server);
   g_clear_object (&self->priv->connection);
   g_clear_object (&self->priv->client);
-  g_clear_object (&self->priv->cancellable);
 }
 
 static void
