@@ -243,7 +243,7 @@ get_identifier (GoaKerberosIdentity  *self,
   krb5_principal principal;
   krb5_error_code error_code;
   char *unparsed_name;
-  char *identifier;
+  char *identifier = NULL;
 
   if (self->priv->credentials_cache == NULL)
     return NULL;
@@ -289,12 +289,14 @@ get_identifier (GoaKerberosIdentity  *self,
       goa_debug ("GoaKerberosIdentity: Error parsing principal identity name: %s",
                error_message);
       krb5_free_error_message (self->priv->kerberos_context, error_message);
-      return NULL;
+      goto out;
     }
 
   identifier = g_strdup (unparsed_name);
   krb5_free_unparsed_name (self->priv->kerberos_context, unparsed_name);
 
+out:
+  krb5_free_principal (self->priv->kerberos_context, principal);
   return identifier;
 }
 
@@ -332,6 +334,7 @@ set_error_from_krb5_error_code (GoaKerberosIdentity  *self,
   va_start (args, format);
   literal_message = g_strdup_vprintf (expanded_format, args);
   va_end (args);
+  g_free (expanded_format);
 
   g_set_error_literal (error, GOA_IDENTITY_ERROR, code, literal_message);
   g_free (literal_message);
